@@ -8,9 +8,10 @@ import threading
 import weakref
 from collections import deque
 from enum import Enum
-from pathlib import Path
+from pathlib import Path, PurePath
 from typing import (
     Any,
+    Callable,
     Deque,
     Dict,
     Generator,
@@ -86,6 +87,7 @@ class State(Enum):
     Stopped = 0
     Running = 1
     Paused = 2
+    CallKeyword = 3
 
 
 class RequestedState(Enum):
@@ -182,11 +184,16 @@ class StackFrameEntry:
 
 
 class HitCountEntry(NamedTuple):
-    source: str
+    source: pathlib.PurePath
     line: int
     type: str
 
 
+class PathMapping(NamedTuple):
+    local_root: Optional[str]
+    remote_root: Optional[str]
+
+
 class Debugger:
     __instance = None
     __lock = threading.RLock()
@@ -217,7 +224,7 @@ class Debugger:
         raise RuntimeError(f"Attempt to create a '{cls.__qualname__}' instance outside of instance()")
 
     def __init__(self) -> None:
-        self.breakpoints: Dict[str, BreakpointsEntry] = {}
+        self.breakpoints: Dict[pathlib.PurePath, BreakpointsEntry] = {}
 
         self.exception_breakpoints: Set[ExceptionBreakpointsEntry] = set()
         self.exception_breakpoints.add(
@@ -242,6 +249,16 @@ class Debugger:
         self.stop_on_entry = False
         self.no_debug = False
         self.terminated = False
+        self.terminated_requested = False
+        self.attached = False
+        self.path_mappings: List[PathMapping] = []
+
+        self._keyword_to_evaluate: Optional[Callable[..., Any]] = None
+        self._evaluated_keyword_result: Any = None
+        self._evaluate_keyword_event = threading.Event()
+        self._evaluate_keyword_event.set()
+        self._after_evaluate_keyword_event = threading.Event()
+        self._after_evaluate_keyword_event.set()
 
     @property
     def debug(self) -> bool:
@@ -271,6 +288,10 @@ class Debugger:
     def robot_output_file(self, value: Optional[str]) -> None:
         self._robot_output_file = value
 
+    @_logger.call
+    def terminate_requested(self) -> None:
+        self.terminated_requested = True
+
     @_logger.call
     def terminate(self) -> None:
         self.terminated = True
@@ -297,11 +318,23 @@ class Debugger:
             self.condition.notify_all()
 
     @_logger.call
-    def continue_thread(self, thread_id: int) -> None:
+    def continue_all(self, send_event: bool = True) -> None:
+        if self.main_thread is not None and self.main_thread.ident is not None:
+            self.continue_thread(self.main_thread.ident, send_event)
+
+    @_logger.call
+    def continue_thread(self, thread_id: int, send_event: bool = False) -> None:
         if self.main_thread is None or thread_id != self.main_thread.ident:
             raise InvalidThreadId(thread_id)
 
         with self.condition:
+            if send_event:
+                self.send_event(
+                    self,
+                    ContinuedEvent(
+                        body=ContinuedEventBody(thread_id=self.main_thread.ident, all_threads_continued=True)
+                    ),
+                )
             self.state = State.Running
             self.condition.notify_all()
 
@@ -399,7 +432,11 @@ class Debugger:
         lines: Optional[List[int]] = None,
         source_modified: Optional[bool] = None,
     ) -> List[Breakpoint]:
-        path = str(Path(source.path).resolve()) if source.path else ""
+
+        if self.is_windows_path(source.path or ""):
+            path: pathlib.PurePath = pathlib.PureWindowsPath(source.path or "")
+        else:
+            path = pathlib.PurePath(source.path or "")
 
         if path in self.breakpoints and not breakpoints and not lines:
             self.breakpoints.pop(path)
@@ -408,7 +445,8 @@ class Debugger:
                 tuple(breakpoints) if breakpoints else (), tuple(lines) if lines else ()
             )
             return [
-                Breakpoint(id=id(v), source=Source(path=path), verified=True, line=v.line) for v in result.breakpoints
+                Breakpoint(id=id(v), source=Source(path=str(path)), verified=True, line=v.line)
+                for v in result.breakpoints
             ]
         else:
             self._logger.error("not supported breakpoint")
@@ -474,9 +512,9 @@ class Debugger:
                 self.requested_state = RequestedState.Nothing
 
         if source is not None:
-            source = str(Path(source).resolve())
-            if source in self.breakpoints:
-                breakpoints = [v for v in self.breakpoints[source].breakpoints if v.line == line_no]
+            source_path = self.map_path_to_client(str(Path(source).absolute()))
+            if source_path in self.breakpoints:
+                breakpoints = [v for v in self.breakpoints[source_path].breakpoints if v.line == line_no]
                 if len(breakpoints) > 0:
                     for point in breakpoints:
                         if point.condition is not None:
@@ -493,7 +531,7 @@ class Debugger:
                                 return
                         if point.hit_condition is not None:
                             hit = False
-                            entry = HitCountEntry(source, line_no, type)
+                            entry = HitCountEntry(source_path, line_no, type)
                             if entry not in self.hit_counts:
                                 self.hit_counts[entry] = 0
                             self.hit_counts[entry] += 1
@@ -519,7 +557,7 @@ class Debugger:
                                     body=OutputEventBody(
                                         output=message + os.linesep,
                                         category=OutputCategory.CONSOLE,
-                                        source=Source(path=source) if source else None,
+                                        source=Source(path=str(source_path)),
                                         line=line_no,
                                     )
                                 ),
@@ -549,6 +587,7 @@ class Debugger:
             )
         ):
             self.state = State.Paused
+
             self.send_event(
                 self,
                 StoppedEvent(
@@ -565,8 +604,27 @@ class Debugger:
 
     @_logger.call
     def wait_for_running(self) -> None:
-        with self.condition:
-            self.condition.wait_for(lambda: self.state in [State.Running, State.Stopped])
+        if self.attached:
+            while True:
+                with self.condition:
+                    self.condition.wait_for(lambda: self.state in [State.Running, State.Stopped, State.CallKeyword])
+
+                if self.state == State.CallKeyword:
+                    self._evaluated_keyword_result = None
+                    try:
+                        if self._keyword_to_evaluate is not None:
+                            self._evaluated_keyword_result = self._keyword_to_evaluate()
+                            self._keyword_to_evaluate = None
+                    except (SystemExit, KeyboardInterrupt):
+                        raise
+                    except BaseException as e:
+                        self._evaluated_keyword_result = e
+                    finally:
+                        self._evaluate_keyword_event.set()
+                        self._after_evaluate_keyword_event.wait(60)
+
+                    continue
+                break
 
     def start_output_group(self, name: str, attributes: Dict[str, Any], type: Optional[str] = None) -> None:
         if self.group_output:
@@ -580,7 +638,7 @@ class Debugger:
                         output=f"\u001b[38;5;14m{(type +' ') if type else ''}\u001b[0m{name}\n",
                         category=OutputCategory.CONSOLE,
                         group=OutputGroup.START,
-                        source=Source(path=source) if source else None,
+                        source=Source(path=str(self.map_path_to_client(source))) if source else None,
                         line=line_no if source is not None else None,
                         column=0 if source is not None else None,
                     )
@@ -599,7 +657,7 @@ class Debugger:
                         output="",
                         category=OutputCategory.CONSOLE,
                         group=OutputGroup.END,
-                        source=Source(path=source) if source else None,
+                        source=Source(path=str(self.map_path_to_client(source))) if source else None,
                         line=line_no,
                     )
                 ),
@@ -861,6 +919,41 @@ class Debugger:
 
         return [Thread(id=main_thread.ident if main_thread.ident else 0, name=main_thread.name or "")]
 
+    WINDOW_PATH_REGEX = re.compile(r"^(([a-z]:[\\/])|(\\\\)).*$", re.RegexFlag.IGNORECASE)
+
+    @classmethod
+    def is_windows_path(cls, path: Union[os.PathLike[str], str]) -> bool:
+        return bool(cls.WINDOW_PATH_REGEX.fullmatch(str(path)))
+
+    @staticmethod
+    def relative_to(path: pathlib.PurePath, *other: pathlib.PurePath) -> Optional[pathlib.PurePath]:
+        try:
+            return path.relative_to(*other)
+        except ValueError:
+            return None
+
+    def map_path_to_client(self, path: Union[os.PathLike[str], str]) -> pathlib.PurePath:
+        if not isinstance(path, PurePath):
+            path = PurePath(path)
+
+        if not self.path_mappings:
+            return path
+
+        for mapping in self.path_mappings:
+
+            remote_root_path = Path(mapping.remote_root or ".").absolute()
+
+            if (
+                mapping.local_root is not None
+                and (relative_path := self.relative_to(Path(path), remote_root_path)) is not None
+            ):
+                if self.is_windows_path(mapping.local_root):
+                    return pathlib.PureWindowsPath(mapping.local_root, relative_path)
+                else:
+                    return pathlib.PurePath(mapping.local_root, relative_path)
+
+        return path
+
     def get_stack_trace(
         self,
         thread_id: int,
@@ -876,7 +969,7 @@ class Debugger:
 
         def source_from_entry(entry: StackFrameEntry) -> Optional[Source]:
             if entry.source is not None and entry.is_file:
-                return Source(path=entry.source)
+                return Source(path=str(self.map_path_to_client(entry.source)))
             else:
                 return None
 
@@ -924,7 +1017,11 @@ class Debugger:
             self.last_fail_message = msg
 
         current_frame = self.full_stack_frames[0] if self.full_stack_frames else None
-        source = Source(path=current_frame.source) if current_frame else None
+        source = (
+            Source(path=str(self.map_path_to_client(current_frame.source)))
+            if current_frame and current_frame.source
+            else None
+        )
         line = current_frame.line if current_frame else None
 
         if self.output_log:
@@ -1142,8 +1239,35 @@ class Debugger:
                         variables.append(var)
 
                     if splitted:
-                        kw = Keyword(name=splitted[0], args=tuple(splitted[1:]), assign=tuple(variables))
-                        result = kw.run(evaluate_context)
+
+                        def run_kw() -> Any:
+                            kw = Keyword(name=splitted[0], args=tuple(splitted[1:]), assign=tuple(variables))
+                            return kw.run(evaluate_context)
+
+                        with self.condition:
+                            self._keyword_to_evaluate = run_kw
+                            self._evaluated_keyword_result = None
+
+                            self._evaluate_keyword_event.clear()
+                            self._after_evaluate_keyword_event.clear()
+
+                            old_state = self.state
+                            self.state = State.CallKeyword
+                            self.condition.notify_all()
+
+                        try:
+                            self._evaluate_keyword_event.wait(60)
+                        finally:
+                            result = self._evaluated_keyword_result
+
+                            with self.condition:
+                                self._keyword_to_evaluate = None
+                                self._evaluated_keyword_result = None
+
+                                self.state = old_state
+                                self.condition.notify_all()
+
+                                self._after_evaluate_keyword_event.set()
 
             elif self.IS_VARIABLE_RE.match(expression.strip()):
                 try:
