@@ -50,7 +50,7 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
         self._workspace_languages: weakref.WeakKeyDictionary[WorkspaceFolder, Languages] = weakref.WeakKeyDictionary()
 
     async def get_workspace_languages(self, document_or_uri: Union[TextDocument, Uri, str]) -> Optional[Languages]:
-        if get_robot_version() < (5, 1):
+        if get_robot_version() < (6, 0):
             return None
 
         from robot.conf.languages import (
@@ -97,7 +97,7 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
     async def build_languages_from_model(
         self, document: TextDocument, model: ast.AST
     ) -> Tuple[Optional[Languages], Optional[Languages]]:
-        if get_robot_version() < (5, 1):
+        if get_robot_version() < (6, 0):
             return (None, None)
 
         from robot.conf.languages import Languages as RobotLanguages
@@ -171,7 +171,7 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
     ) -> Any:
         import robot.api
 
-        if get_robot_version() >= (5, 1):
+        if get_robot_version() >= (6, 0):
             return robot.api.get_tokens(source, data_only=data_only, tokenize_variables=tokenize_variables, lang=lang)
         else:
             return robot.api.get_tokens(source, data_only=data_only, tokenize_variables=tokenize_variables)
@@ -185,7 +185,7 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
     ) -> Any:
         import robot.api
 
-        if get_robot_version() >= (5, 1):
+        if get_robot_version() >= (6, 0):
             return robot.api.get_resource_tokens(
                 source, data_only=data_only, tokenize_variables=tokenize_variables, lang=lang
             )
@@ -197,7 +197,7 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
     ) -> Any:
         import robot.api
 
-        if get_robot_version() >= (5, 1):
+        if get_robot_version() >= (6, 0):
             return robot.api.get_init_tokens(
                 source, data_only=data_only, tokenize_variables=tokenize_variables, lang=lang
             )
@@ -228,7 +228,7 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
         get: Callable[[str], List[Token]],
     ) -> List[Token]:
 
-        return get(document.text)
+        return get(await document.text())
 
     async def get_resource_tokens(self, document: TextDocument, data_only: bool = False) -> List[Token]:
         if data_only:
@@ -442,10 +442,10 @@ class DocumentsCache(RobotLanguageServerProtocolPart):
             else:
                 return await self.default_imports_manager()
 
-        if folder not in self._imports_managers:
-            async with self._imports_managers_lock:
-                if folder not in self._imports_managers:
-                    config = await self.parent.workspace.get_configuration(RobotConfig, folder.uri)
+        async with self._imports_managers_lock:
+            if folder not in self._imports_managers:
+                config = await self.parent.workspace.get_configuration(RobotConfig, folder.uri)
 
-                    self._imports_managers[folder] = ImportsManager(self.parent, folder.uri, config)
-        return self._imports_managers[folder]
+                self._imports_managers[folder] = ImportsManager(self.parent, folder.uri, config)
+
+            return self._imports_managers[folder]
